"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const escape_string_regexp_1 = __importDefault(require("escape-string-regexp"));
const openapi_1 = require("../../utils/openapi");
const utils_1 = require("../../utils");
const name = 'latin-definitions-only';
function replaceLatinCharacters(str) {
    return (str
        // def name may contain latin chars
        .replace(/[a-z]+/gi, '')
        // or numerals
        .replace(/\d/gi, ''));
}
function replaceIgnoredChars(str, ignoredChars) {
    if (ignoredChars.length === 0)
        return str;
    const re = new RegExp(`[${ignoredChars.map(x => escape_string_regexp_1.default(x)).join('')}]`, 'g');
    return str.replace(re, '');
}
const rule = utils_1.createRule({
    name,
    docs: {
        recommended: true,
        description: 'Bans non Latin characters usage in definition names',
    },
    meta: {
        messages: {
            msg: `Definition name "{{name}}" contains non latin characters.`,
        },
        schema: {
            type: 'array',
            items: [
                {
                    type: 'string',
                },
                {
                    type: 'object',
                    required: ['ignore'],
                    properties: {
                        ignore: {
                            type: 'array',
                            items: {
                                type: 'string',
                                minLength: 1,
                                maxLength: 1,
                            },
                        },
                    },
                },
            ],
            minItems: 1,
            maxItems: 2,
        },
    },
    swaggerVisitor: {
        DefinitionsObject: ({ node, report, location, setting }) => {
            var _a;
            const charsToIgnore = (Array.isArray(setting)
                ? ((_a = setting[1]) === null || _a === void 0 ? void 0 : _a.ignore) || []
                : []);
            const definitionNames = Object.keys(node);
            definitionNames.forEach(name => {
                const cleanStr = replaceLatinCharacters(name);
                const rest = replaceIgnoredChars(cleanStr, charsToIgnore);
                if (rest.length > 0) {
                    report({
                        messageId: 'msg',
                        data: {
                            name,
                        },
                        location: [...location, name],
                    });
                }
            });
        },
    },
    openapiVisitor: {
        ComponentsObject: ({ node, location, report, setting }) => {
            var _a;
            const charsToIgnore = (Array.isArray(setting)
                ? ((_a = setting[1]) === null || _a === void 0 ? void 0 : _a.ignore) || []
                : []);
            openapi_1.componentsKeys.forEach(compName => {
                const val = node[compName];
                if (val === undefined)
                    return;
                Object.keys(val).forEach(recName => {
                    const cleanStr = replaceLatinCharacters(recName);
                    const rest = replaceIgnoredChars(cleanStr, charsToIgnore);
                    if (rest.length > 0) {
                        report({
                            messageId: 'msg',
                            data: {
                                name: recName,
                            },
                            location: [...location, compName, recName],
                        });
                    }
                });
            });
        },
    },
    defaultSetting: ['placeholder_to_be_removed', { ignore: [] }],
});
exports.default = rule;
