"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const case_1 = __importDefault(require("case"));
const utils_1 = require("../../utils");
const name = 'parameter-casing';
function getCasesSetFromOptions(option, defaultCase) {
    return typeof option === 'string' && utils_1.isValidCaseName(option)
        ? utils_1.validCases[option]
        : defaultCase;
}
const PARAMETER_LOCATIONS = [
    'query',
    'header',
    'path',
    'cookie',
    'formData',
    'body',
];
const validCasesArr = Object.keys(utils_1.validCases);
const paramsSchema = PARAMETER_LOCATIONS.reduce((acc, el) => {
    acc[el] = {
        type: 'string',
        enum: validCasesArr,
    };
    return acc;
}, {});
const rule = utils_1.createRule({
    name,
    docs: {
        recommended: true,
        description: 'Casing for your parameters',
    },
    meta: {
        messages: {
            casing: 'Parameter "{{name}}" has wrong casing. Should be "{{correctVersion}}".',
        },
        schema: {
            type: 'array',
            items: [
                {
                    type: 'string',
                    enum: Object.keys(utils_1.validCases),
                },
                {
                    type: 'object',
                    properties: Object.assign(Object.assign({}, paramsSchema), { ignore: {
                            type: 'array',
                            items: {
                                type: 'string',
                            },
                        } }),
                    additionalProperties: false,
                },
            ],
            minItems: 1,
            maxItems: 2,
        },
    },
    swaggerVisitor: {
        ParameterObject: ({ node, report, location, setting }) => {
            if (typeof setting === 'boolean')
                return;
            const [settingCasingName, opts = {}] = setting;
            if (typeof settingCasingName === 'string' &&
                utils_1.isValidCaseName(settingCasingName)) {
                const defaultParamCase = utils_1.validCases[settingCasingName];
                const cases = {
                    query: getCasesSetFromOptions(opts.query, defaultParamCase),
                    header: getCasesSetFromOptions(opts.header, defaultParamCase),
                    path: getCasesSetFromOptions(opts.path, defaultParamCase),
                    formData: getCasesSetFromOptions(opts.formData, defaultParamCase),
                    body: getCasesSetFromOptions(opts.body, defaultParamCase),
                };
                const IGNORE_PARAMETER_NAMES = new Set(Array.isArray(opts.ignore) ? opts.ignore : []);
                if (IGNORE_PARAMETER_NAMES.has(node.name))
                    return;
                const nodeCase = case_1.default.of(node.name);
                const paramLocation = node.in;
                if (!cases[paramLocation].has(nodeCase)) {
                    const shouldBeCase = cases[paramLocation]
                        .values()
                        .next().value;
                    const correctVersion = case_1.default[shouldBeCase](node.name);
                    report({
                        messageId: 'casing',
                        data: {
                            name: node.name,
                            correctVersion,
                        },
                        location: [...location, 'name'],
                    });
                }
            }
        },
    },
    openapiVisitor: {
        ParameterObject: ({ node, report, location, setting }) => {
            if (typeof setting === 'boolean')
                return;
            const [settingCasingName, opts = {}] = setting;
            if (typeof settingCasingName === 'string' &&
                utils_1.isValidCaseName(settingCasingName)) {
                const defaultParamCase = utils_1.validCases[settingCasingName];
                const cases = {
                    query: getCasesSetFromOptions(opts.query, defaultParamCase),
                    header: getCasesSetFromOptions(opts.header, defaultParamCase),
                    path: getCasesSetFromOptions(opts.path, defaultParamCase),
                    cookie: getCasesSetFromOptions(opts.cookie, defaultParamCase),
                };
                const IGNORE_PARAMETER_NAMES = new Set(Array.isArray(opts.ignore) ? opts.ignore : []);
                if (IGNORE_PARAMETER_NAMES.has(node.name))
                    return;
                const nodeCase = case_1.default.of(node.name);
                const paramLocation = node.in;
                if (!cases[paramLocation].has(nodeCase)) {
                    const shouldBeCase = cases[paramLocation]
                        .values()
                        .next().value;
                    const correctVersion = case_1.default[shouldBeCase](node.name);
                    report({
                        messageId: 'casing',
                        data: {
                            name: node.name,
                            correctVersion,
                        },
                        location: [...location, 'name'],
                    });
                }
            }
        },
    },
    defaultSetting: ['camel', { header: 'kebab' }],
});
exports.default = rule;
